open Lexing
open Format
open Options

module Time = struct

  open Unix
    
  let u = ref 0.0
    
  let start () = u:=(times()).tms_utime

  let get () = (times()).tms_utime -. !u

end

module Sat = Sat

let print_status fmt s = 
  fprintf fmt "%s"
    ((function 
	  Smt_ast.Sat ->  "sat" 
	| Smt_ast.Unsat -> "unsat" 
	| Smt_ast.Unknown -> "unknown") s)

let process_decl env d =
  let satmode = smtfile or satmode in
  let loc = match d with 
      Why_typing.Assume(_,l) | Why_typing.Query(_,_,l) ->l in
  try
    match d with
	Why_typing.Assume(f,loc) -> Sat.assume env (f,0,None)
      | Why_typing.Query(n,f,loc) -> 
	  let f = if satmode then f else Formula.mk_not f in
	  Sat.unsat env (f,0,None) stopb; 
	  Loc.report loc;
	  if satmode then printf "Unsat.@." 
	  else printf "Valid (%2.4f)@." (Time.get());
	  env
  with 
    | Sat.Sat -> 
	Loc.report loc;
	if satmode then printf "sat.@." else printf "Invalid@."; env
    | Sat.Unsat -> 
	Loc.report loc; 
	printf "Invalid (possible incompleteness with quantifiers).\n@." ; env
    | Sat.I_dont_know -> 
	Loc.report loc; printf "I don't know.\n@." ; env

let main () = 
  let c = open_in file in
  let lb = from_channel c in
  try
    let d , status =
      if smtfile then begin
	let lb_prelude = 
	  from_channel (open_in (Version.libdir^"/ergo/smt_prelude.mlw")) in
	let lp = Why_parser.file Why_lexer.token lb_prelude in
        let bname,l,status = Smt_parser.benchmark Smt_lex.token lb in
	if verbose then printf "converting smt file@.";
        let l = List.flatten (List.map Smt_to_why.bench_to_why l) in
         Why_typing.file (lp@l) , status
      end
      else
	(let a = Why_parser.file Why_lexer.token lb in
	 if parse_only then exit 0;
         Why_typing.file a) , Smt_ast.Unknown
    in
    if smtfile then printf "status:%a   " print_status status;
    close_in c;
    if type_only then exit 0;
    ignore(Queue.fold process_decl Sat.empty d)
(*    exit 0*)
  with
    | Why_lexer.Lexical_error s -> 
	Loc.report (lexeme_start_p lb, lexeme_end_p lb);
	printf "lexical error: %s\n@." s;
	exit 1
    | Parsing.Parse_error ->
	let  loc = (lexeme_start_p lb, lexeme_end_p lb) in
	Loc.report loc;
        printf "syntax error\n@.";
	exit 1
    | Why_typing.Error(e,l) -> 
	Loc.report l; 
	printf "typing error: %a\n@." Why_typing.report e;
	exit 1

let _ = Time.start (); main ()
(*  try main ()
  with e -> printf "%s@." (Printexc.to_string e); exit 2*)




  

