module type S = sig
  type t

  module R : Sig.THEORY
  module Res : Set.S with type elt = R.t * Term.Set.t * R.Set.t

  exception Unknown_element
  exception Inconsistent

  val empty :  t
  val add : Term.t -> t -> t
  val mem : Term.t -> t -> bool
  val find : t -> Term.t -> R.t
  val union : t -> Term.t -> Term.t -> Formula.Set.t -> bool ->  t * Res.t
  val distinct : t -> Term.t -> Term.t -> Formula.Set.t -> t

  val leaves : t -> Term.t -> R.Set.t
  val equal : t -> Term.t -> Term.t -> bool
  val are_distinct : t -> Term.t -> Term.t -> bool
  val classof : t -> Term.t -> Term.t list

  val explain : t -> Term.t -> Term.t -> Formula.Set.t
  val nexplain : t -> Term.t -> Term.t -> Formula.Set.t

end

module Make ( X : Sig.THEORY ) : S with module R = X
